var localVideo = document.getElementById('localVideo');
var canvas = document.getElementById('canvas');
var custom = document.getElementById('custom');
var customUrl = document.getElementById('customUrl');
var boardTool = document.getElementById('board-tool');
var boardSetBtn = document.getElementById('boardSetBtn');
var createBoardBtn = document.getElementById('createBoard');
var clearBoardBtn = document.getElementById('clearBoard');
var updateBgBtn = document.getElementById('updateBoard');
var boardColorSet = document.getElementById('boardColorSet');
var receiverA = document.getElementById('receiverBtn');
var updateBoardBgColorBtn = document.getElementById('updateBoardBgColor');
var chooseBgColorBtn = document.getElementById('chooseBgColor');
var imageURL,localVideoId;
var imgForCanvas = new Image();


var self = this;

// 获取token
restApi.getAccessToken(accessKey, secretKey).then((res) => {
    log.debug('Success', res)
    self.accessToken = res.token;
}).otherwise((error) => {
    log.debug("Error", error);
    $(".promptMsg").html('getAccessToken(),error:' + error.msg);

})

//  加入会议
function joinRoom() {
    if (room) { return }
    if ($("#first").val() == '') {
        $('.promptMsg').html('会议号不能为空，请先创建房间');
    } else {
        var receiverA = document.getElementById('receiverBtn');
        self.roomId = $("#first").val();
        if(receiverA) {
            receiverA.href = './reception.html?roomId=' + self.roomId;
        }
        avdEngine.init(self.serverURL, self.accessToken).then(initSuccess).otherwise(initError);
    }
}

//  初始化成功
function initSuccess() {
    room = avdEngine.obtainRoom(self.roomId);
    room.join(self.userId, self.userName, '', '').then(joinSuccess).otherwise(joinError);
}

//  初始化失败
function initError(error) {
    $(".promptMsg").html("AVDEngine 初始化失败！initError(),error:" + error.message);
}

//  加入房间失败
function joinError(error) {
    console.log('==================joinError(),error:', error)
    if (error.code == 404) {
        $(".promptMsg").html("加会失败！原因房间没有找到，请核实。joinError(),error");
    } else {
        $(".promptMsg").html('joinError(),error:' + error.message + "[" + error.code + "]");
    }
}

//创建房间
function createRoom() {
    var topic = 'testingMeeting';
    restApi.createRoom(self.accessToken, topic, self.userId).then((res) => {
        console.log('Success', res)
        $("#first").val(res.room_id);
    }).otherwise((error) => {
        console.log("Error", error)
        $(".promptMsg").html('createRoom(),error:' + error.msg);
    })
}

//  加入房间成功
function joinSuccess() {
    console.log('加入会议成功');
    $(".promptMsg").html('加入会议成功');
    setTimeout(()=>{
        $(".promptMsg").html('');
    },2000)
    boardSetBtn.disabled = false;
    createBoardBtn.disabled = false;
    receiverA.style.display = 'block';

    registerRoomCallback();
}

function registerRoomCallback() {
    room.addCallback(RoomCallback.connection_status, onConnectionStatus);
}

/**
 * @desc 网络状态回调
 */
function onConnectionStatus(status) {
    if(status == ConnectionStatus.connecting) {
		$(".promptMsg").html("网络故障,正在与服务器重连中...");
	} else if(status == ConnectionStatus.connected) {
		//连接成功
		$(".promptMsg").html("");
	} else if(status == ConnectionStatus.reJoinConnected) {
		//重新加会成功
		$(".promptMsg").html("");
	} else if(status == ConnectionStatus.reconnected) {
		//重连接成功
		$(".promptMsg").html("");
	} else if(status == ConnectionStatus.connectFailed) {
		$(".promptMsg").html("网络故障,与服务器重连超时，正在重新加会操作中...");

		//应用层清场
		if(room.connectionInfoCollector) {
			room.connectionInfoCollector.stop(); //停止收集网络情况
		}
	} else if(status == ConnectionStatus.reJoinRoomTimeOut) {
		$(".promptMsg").html("网络故障,重新加会中...");
	    room.continuousReJoin();
	}
}


//打开摄像头 
function openCamera() {
    var videos = room.selfUser.videos;
    if (videos && videos.length > 0) {
        var video = videos[0];
        if (video) {
            localVideoId = video.id;
            video.previewAndPublish(localVideo).then(()=>{
                setTimeout(()=>{
                    canvas.width = localVideo.offsetWidth;
                    canvas.height = localVideo.offsetHeight;
                    canvas.getContext('2d').drawImage(localVideo, 0, 0, canvas.width, canvas.height);
                    saveAndUpload();
                },1000)
            }).otherwise(showError);
        }
    }
}


//关闭摄像头
function closeCamera(){
    var video = room.selfUser.getVideo(localVideoId);
    console.log
	video.unpreview();
	video.unpublish();
}

function showError() {
    $(".promptMsg").html('showError(),error:'+error.message+"["+error.code+"]");
}

var boardDom = document.getElementById('board_board');
// 创建白板
function createBoard(customUrl){
    console.log('+++123, createBoard()');
    if(customUrl) {
        var backgroundImage = customUrl;
    } else {
        var backgroundImage = imageURL;
    }

    if(boardColorSet.checked) {
        var backgroundColor = colorRgb(boardColorIpt.value);
        // 如果设置了底色就不设置图片
        backgroundImage = null;
    } else {
        var backgroundColor = '0,0,0,0';
    }
    var boardWidth = boardDom.offsetWidth;
    var boardHeight = boardDom.offsetHeight;
    log.info(`+++board: width: ${boardWidth}, height: ${boardHeight}`)
   
    var board = room.selfUser.createBoard(boardWidth, boardHeight, backgroundColor, backgroundImage,'',800,800);
    createBoardHandle(board);
}

/**
 * board对象的内容
 * @param {String} userId - 创建者userId
 * @param {int} width -  白板的渲染宽度
 * @param {int} height - 白板的渲染高度
 * @param {Object} backgroundColor - 白板的背景色
 * @param {String} backgroundImage - 白板的背景图访问路径
 * @param {String} title - 白标标题
 * @param {int} outputWidth - 白板的输出宽度
 * @param {int} outputHeight - 白板的输出高度
 * @param {String} description - 描述
 * @param {String} extendData - 扩展内容
 */
function createBoardHandle(board) {
    console.log('+++123, createBoardHandle()');
    
    currentBoard = board;
    
	var newBoardDiv = document.createElement("Div");
	newBoardDiv.id = "board_" + board.id;
	newBoardDiv.style.width = board.width + "px";
	newBoardDiv.style.height = board.height + "px";
    newBoardDiv.style.position = 'relative';
    newBoardDiv.className = 'newBoardDiv';
    // newBoardDiv.style.background = 'url('+board.backgroundImage+') no-repeat center center';
    newBoardDiv.style.backgroundSize = '100% 100%';
    newBoardDiv.style.border = '5px solid #222';
    newBoardDiv.style.boxSizing = 'border-box';

	newBoardDiv.boardUserId = board.userId;
	newBoardDiv.boardId = board.id;

    var newBoardCanvas = document.createElement("Div");
    newBoardCanvas.className = 'newBoardCanvas';
	newBoardCanvas.style.width = "100%";
	newBoardCanvas.style.height = "100%";
	// newBoardCanvas.style.background = "rgba(" + board.backgroundColor + ")";
	newBoardCanvas.style.position = 'relative';

    // 创建批注得背景canvas
    var backgroundCanvas = document.createElement("canvas");
    backgroundCanvas.id = 'backgroundCanvas';
    backgroundCanvas.width = (board.width - 10);
	backgroundCanvas.height = (board.height - 10);
    backgroundCanvas.style.position = 'absolute';
    backgroundCanvas.style.left = '0';
    backgroundCanvas.style.top = '0';
    backgroundCanvas.style.zIndex = '-1';
    backgroundCanvas.getContext('2d').fillStyle = "rgba(" + board.backgroundColor + ")";
    backgroundCanvas.getContext('2d').fillRect(0, 0, backgroundCanvas.width, backgroundCanvas.height);
    
    // 设置背景canvas的img属性
    imgForCanvas.crossOrigin = 'anonymous';

    
    newBoardDiv.appendChild(newBoardCanvas);
    newBoardDiv.appendChild(backgroundCanvas);
	boardDom.appendChild(newBoardDiv);
    
    if(board.backgroundImage){
        updateImgAndCanvas(board.backgroundImage + '?temp=' + new Date().getTime());
    }

	board.createAnnotation().then(function(annotation) {
		annotation.init(newBoardCanvas)
		board.annotation.hlightPointInit('./img/icon-laserPen-move.png', 30, 30);
    });
    
    boardTool.style.display = 'flex';
    clearBoardBtn.disabled = false;
    createBoardBtn.disabled = true;
    boardSetBtn.disabled = true;
    updateBgBtn.disabled = false;
    chooseBgColorBtn.disabled = false;
    updateBoardBgColorBtn.disabled = false;
    if(boardColorSet.checked){
        updateBgBtn.disabled = true;
    }
}

//点击进行截图，需要传入video元素
function screenshots() {
    console.log('+++123, screenshots()');
    if(custom.checked || boardColorSet.checked) {
        createBoard(customUrl.value);
    } else {
        //  打开视频
        openCamera();
    }
}


function videoCapture(){
    if(room && room.selfUser.boards[0]){
        var videoDom = document.getElementById('localVideo');
        canvas.width = videoDom.offsetWidth;
        canvas.height = videoDom.offsetHeight;
        canvas.getContext('2d').drawImage(videoDom, 0, 0, canvas.width, canvas.height);
        saveAndUpload();
    }
}


/**
 * @desc 更新canvas背景层的图片
 * @param {String} imgUrl 图片路径
 */
function updateImgAndCanvas(imgUrl){
    console.log('+++123, updateImgAndCanvas begin, url: ', imgUrl);
    var bgCtx = document.getElementById('backgroundCanvas').getContext('2d');
    var board = room.selfUser.boards[0];
    imgForCanvas.src = imgUrl;
    imgForCanvas.onload = function (){
        bgCtx.drawImage(imgForCanvas, 0, 0, imgForCanvas.width, imgForCanvas.height, 0, 0, board.width - 10, board.height - 10);
        console.log('+++123, updateImgAndCanvas finished');
    }
}

/**
 * @desc 更改白板背景色
 */
function updateBoardBgColor(){
    var backgroundCanvas = document.getElementById('backgroundCanvas');
    var backgroundCanvasCtx = backgroundCanvas.getContext('2d');
    var bgValue = document.getElementById('chooseBgColor').value;
    backgroundCanvasCtx.fillStyle = bgValue;
    backgroundCanvasCtx.fillRect(0, 0, backgroundCanvas.width, backgroundCanvas.height);
    currentBoard.updateBackgroundColor(colorRgb(bgValue));
    // 更改背景色，就不设置背景图
    currentBoard.updateBackgroundImage(null);
    room.selfUser.updateBoard(currentBoard);
}

//保存并上传图片
function saveAndUpload() {
    console.log('+++123, saveAndUpload()');
    var strDataURI = canvas.toDataURL("image/png").replace("image/png", "image/octet-stream");
    var test = dataURLtoFile(strDataURI, 'test.png');
    console.log('11111111111111',test);
    changefile(test);
}

function changefile(file, room_id){
    console.log('+++123, changefile()');
    var forms = new FormData()
    var configs = {
      headers:{'Content-Type':'multipart/form-data'}
    };
    
    forms.append('file',file);

    if(!room_id) {
    	room_id = roomId;
    }
    var url = document.location.protocol + '//' + serverURL + '/sdkapi/v2/file/upload_file?token=' +  accessToken + '&room_id=' + room_id + '&user_id=' + userId;
    
    
    this.axios.post(url ,forms ,configs).then((res)=>{
        console.log('WWWWWWWWWWWWWWWW11111111111',res);
        if(res.data.ret == 0) {
            imageURL = res.data.file_path;
            console.log('+++123, imageUrl: ', imageURL);
            if(room.selfUser.boards[0]){
                console.log('+++123, user has board');
                if(imageURL){
                    var newBoardDiv = document.getElementsByClassName('newBoardDiv')[0];
                    // 更新白板背景canvas
                    updateImgAndCanvas(imageURL + '?temp=' + new Date().getTime());
                    newBoardDiv.style.backgroundSize = '100% 100%';      
                    if(currentBoard){
                        currentBoard.updateBackgroundImage(imageURL);      
                        room.selfUser.updateBoard(currentBoard);
                    }
            
                }
            }
            else{
                createBoard();
            }
        } else {
            console.log('==========changefile()上传失败' + res.data.msg + '[' + res.data.ret + ']')
        }
    }).catch((error) => {
        console.log('==========changefile()上传失败error:' + error)
    })
}

function dataURLtoFile(dataurl, filename) {
    var arr = dataurl.split(','), mime = arr[0].match(/:(.*?);/)[1],
    bstr = atob(arr[1]), n = bstr.length, u8arr = new Uint8Array(n);
    while(n--){
      u8arr[n] = bstr.charCodeAt(n);
    }
    return new File([u8arr], filename, {type:mime});
}

function closeBoard() {
    var board = room.selfUser.boards[0];
	if(board && board.status == boardStatusEnum.open) {
        if(localVideoId) {
            closeCamera();
        }
        removeBoardHandle(room.selfUser.boards[0].id);
        room.selfUser.closeBoardById(room.selfUser.boards[0].id);
        room.selfUser.boards = [];
        boardTool.style.display = 'none';
	}
}

//退会
function leaveRoom () {
    if(!room){return}
    var close = window.confirm('您确认退出会议?');
	if(close) {
		var reason = 1; //退会原因
		room.leave(reason).then(function() {
            top.location = "index.html";
			return false;
       });
	}
}

function boardSet(){
    var boardSetDom = document.querySelector('.boardSetDom');
    if(boardSetDom.style.display == 'none') {
        boardSetDom.style.display = 'block';
    } else {
        boardSetDom.style.display = 'none';
    }
}

function setBoardColor(){
    var boardColorIpt = document.getElementById('boardColorIpt');
    $('#boardColor').html('rgba('+ colorRgb(boardColorIpt.value) + ')');
}

function save(){
    var boardSetDom = document.querySelector('.boardSetDom');
    boardSetDom.style.display = 'none';
}

function removeBoardHandle(boardId) {
	var removeBoardDivName = "board_" + boardId;
	var removeBoardsDiv = document.getElementById(removeBoardDivName);
	boardDom.removeChild(removeBoardsDiv);
    currentBoard = null;
    
    boardSetBtn.disabled = false;
    createBoardBtn.disabled = false;
}