function RestApi(serverURI) {
    axios.defaults.timeout = 5000;
    axios.defaults.baseURL =   'https://' + serverURI + '/avd2api/v1';

    function fetchGet(url, params) {
        var deferred = when.defer();
        axios.get(url, params).then(function (response) {
            if (response.data.ret === 0) {
                deferred.resolve(response.data);
            } else {
                deferred.reject(response.data);
            }
        }).catch(function (error) {
            if (error.response) {
                // 服务器返回正常的异常对象
                var data = {
                    ret: error.response.status,
                    msg: error.response.data
                }
                deferred.reject(data);
            } else {
                // 服务器发生未处理的异常
                var data = {
                    ret: 10000,
                    msg: error.message,
                }
                deferred.reject(data);
            }
            deferred.reject(error)
        })
        return deferred.promise;
    }

    /**
     * 登录
     * module：授权模块
     * 访问URL: auth/login
     * 错误码区间:10100-10199
     */
    this.login = function (email, password) {
        return fetchGet('/auth/login', { params: { "email": email, "password": password } });
    }

    this.createUser = function (token, name, email, password, room_id, is_have_host) {
        return fetchGet('/user/create', { params: { "token": token, "name": name, "email": email, "password": password, "room_id": room_id, "is_have_host": is_have_host } });
    }
    
    this.getRoomList = function (user_id, session_id) {
        return fetchGet('/room/list', { params: { "user_id": user_id, "session_id": session_id } })
    }

    this.updateMettingList = function (room_id, session_id, start_time, end_time, user_id, topic, description, attendee_max, invite_user_id, is_have_host) {
        var paramsObj = {
            "room_id": room_id,
            "session_id": session_id,
            "start_time": start_time,
            "end_time": end_time,
            "user_id": user_id,
            "topic": topic,
            "description": description,
            "attendee_max": attendee_max,
            "invite_user_id": invite_user_id,
            "is_have_host": is_have_host
        }
        return fetchGet('/room/arrange', { params: paramsObj });
    }

    this.getToken = function (access_key, secret_key) {
        return fetchGet('/auth/get_token', {
            params: {
                'access_key': access_key,
                'secret_key': secret_key
            }
        })
    }

}
